@@ -1,5 +1,5 @@
 /* agent.c - Talking to gpg-agent.
-   Copyright (C) 2006, 2007 g10 Code GmbH
+   Copyright (C) 2006, 2007, 2008 g10 Code GmbH
 
    This file is part of Scute.
  
@@ -38,6 +38,14 @@
 #include <string.h>
 #include <stdarg.h>
 
+#ifdef HAVE_W32_SYSTEM
+#define PATHSEP_C ';'
+#define WINVER 0x0500  /* Required for AllowSetForegroundWindow.  */
+#include <windows.h>
+#else
+#define PATHSEP_C ':'
+#endif
+
 #include <assuan.h>
 #include <gpg-error.h>
 
@@ -49,49 +57,302 @@
 /* The global agent context.  */
 static assuan_context_t agent_ctx = NULL;
 
+
+/* Hack required for Windows.  */
+void 
+gnupg_allow_set_foregound_window (pid_t pid)
+{
+  if (!pid || pid == (pid_t)(-1))
+    return;
+#ifdef HAVE_W32_SYSTEM
+  else if (!AllowSetForegroundWindow (pid))
+    DEBUG ("AllowSetForegroundWindow(%lu) failed: %i\n",
+	   (unsigned long)pid, GetLastError ());
+#endif
+}
+
+
+#ifdef HAVE_W32_SYSTEM
+/* Helper function to build_w32_commandline. */
+static char *
+build_w32_commandline_copy (char *buffer, const char *string)
+{
+  char *p = buffer;
+  const char *s;
+
+  if (!*string) /* Empty string. */
+    p = stpcpy (p, "\"\"");
+  else if (strpbrk (string, " \t\n\v\f\""))
+    {
+      /* Need top do some kind of quoting.  */
+      p = stpcpy (p, "\"");
+      for (s=string; *s; s++)
+        {
+          *p++ = *s;
+          if (*s == '\"')
+            *p++ = *s;
+        }
+      *p++ = '\"';
+      *p = 0;
+    }
+  else
+    p = stpcpy (p, string);
+
+  return p;
+}
+
+
+/* Build a command line for use with W32's CreateProcess.  On success
+   CMDLINE gets the address of a newly allocated string.  */
+static gpg_error_t
+build_w32_commandline (const char *pgmname, const char * const *argv, 
+                       char **cmdline)
+{
+  int i, n;
+  const char *s;
+  char *buf, *p;
+
+  *cmdline = NULL;
+  n = 0;
+  s = pgmname;
+  n += strlen (s) + 1 + 2;  /* (1 space, 2 quoting */
+  for (; *s; s++)
+    if (*s == '\"')
+      n++;  /* Need to double inner quotes.  */
+  for (i=0; (s=argv[i]); i++)
+    {
+      n += strlen (s) + 1 + 2;  /* (1 space, 2 quoting */
+      for (; *s; s++)
+        if (*s == '\"')
+          n++;  /* Need to double inner quotes.  */
+    }
+  n++;
+
+  buf = p = malloc (n);
+  if (!buf)
+    return gpg_error_from_syserror ();
+
+  p = build_w32_commandline_copy (p, pgmname);
+  for (i=0; argv[i]; i++) 
+    {
+      *p++ = ' ';
+      p = build_w32_commandline_copy (p, argv[i]);
+    }
+
+  *cmdline= buf;
+  return 0;
+}
+
+
+/* Spawn a new process and immediatley detach from it.  The name of
+   the program to exec is PGMNAME and its arguments are in ARGV (the
+   programname is automatically passed as first argument).  An error
+   is returned if pgmname is not executable; to make this work it is
+   necessary to provide an absolute file name.  All standard file
+   descriptors are connected to /dev/null.  */
+static gpg_error_t
+spawn_process_detached (const char *pgmname, const char *argv[])
+{
+  gpg_error_t err;
+  SECURITY_ATTRIBUTES sec_attr;
+  PROCESS_INFORMATION pi = 
+    {
+      NULL,      /* Returns process handle.  */
+      0,         /* Returns primary thread handle.  */
+      0,         /* Returns pid.  */
+      0          /* Returns tid.  */
+    };
+  STARTUPINFO si;
+  int cr_flags;
+  char *cmdline;
+
+  if (access (pgmname, X_OK))
+    return gpg_error_from_syserror ();
+
+  /* Prepare security attributes.  */
+  memset (&sec_attr, 0, sizeof sec_attr );
+  sec_attr.nLength = sizeof sec_attr;
+  sec_attr.bInheritHandle = FALSE;
+  
+  /* Build the command line.  */
+  err = build_w32_commandline (pgmname, argv, &cmdline);
+  if (err)
+    return err; 
+
+  /* Start the process.  */
+  memset (&si, 0, sizeof si);
+  si.cb = sizeof (si);
+  si.dwFlags = STARTF_USESHOWWINDOW;
+  si.wShowWindow = SW_MINIMIZE;
+
+  cr_flags = (CREATE_DEFAULT_ERROR_MODE
+              | GetPriorityClass (GetCurrentProcess ())
+              | CREATE_NEW_PROCESS_GROUP
+              | DETACHED_PROCESS); 
+  DEBUG ("CreateProcess(detached), path=`%s' cmdline=`%s'\n",
+	 pgmname, cmdline);
+  if (!CreateProcess (pgmname,       /* Program to start.  */
+                      cmdline,       /* Command line arguments.  */
+                      &sec_attr,     /* Process security attributes.  */
+                      &sec_attr,     /* Thread security attributes.  */
+                      FALSE,         /* Inherit handles.  */
+                      cr_flags,      /* Creation flags.  */
+                      NULL,          /* Environment.  */
+                      NULL,          /* Use current drive/directory.  */
+                      &si,           /* Startup information. */
+                      &pi            /* Returns process information.  */
+                      ))
+    {
+      DEBUG ("CreateProcess(detached) failed: %i\n", GetLastError ());
+      free (cmdline);
+      return gpg_error (GPG_ERR_GENERAL);
+    }
+  free (cmdline);
+  cmdline = NULL;
+
+  DEBUG ("CreateProcess(detached) ready: hProcess=%p hThread=%p"
+	 " dwProcessID=%d dwThreadId=%d\n",
+	 pi.hProcess, pi.hThread,
+	 (int) pi.dwProcessId, (int) pi.dwThreadId);
+
+  CloseHandle (pi.hThread); 
+
+  return 0;
+}
+#endif
+
 
 /* Establish a connection to a running GPG agent.  */
 static gpg_error_t
 agent_connect (assuan_context_t *ctx_r)
 {
+  /* If we ever failed to connect via a socket we will force the use
+     of the pipe based server for the lifetime of the process.  */
+  static int force_pipe_server = 0;
+
   gpg_error_t err = 0;
   char *infostr;
   char *ptr;
-  int pid;
-  int protocol_version;
 
-  infostr = getenv ("GPG_AGENT_INFO");
-  if (!infostr)
+ restart:
+
+  infostr = force_pipe_server ? NULL : getenv ("GPG_AGENT_INFO");
+  if (!infostr || !*infostr)
     {
-      DEBUG ("missing GPG_AGENT_INFO environment variable");
-      return gpg_error (GPG_ERR_NO_AGENT);
+      char *sockname;
+
+      /* First check whether we can connect at the standard
+         socket.  */
+      sockname = make_filename (default_homedir (), "S.gpg-agent", NULL);
+      if (! sockname)
+	return gpg_error_from_errno (errno);
+
+      err = assuan_socket_connect (ctx_r, sockname, 0);
+
+      if (err)
+        {
+	  const char *agent_program;
+
+          /* With no success start a new server.  */
+	  DEBUG ("no running GPG agent at %s, starting one\n", sockname);
+          
+          agent_program = get_gpg_agent_path ();
+
+#ifdef HAVE_W32_SYSTEM
+          {
+            /* Under Windows we start the server in daemon mode.  This
+               is because the default is to use the standard socket
+               and thus there is no need for the GPG_AGENT_INFO
+               envvar.  This is possible as we don't have a real unix
+               domain socket but use a plain file and thus there is no
+               need to care about non-local file systems. */
+            const char *argv[3];
+
+            argv[0] = "--daemon";
+            argv[1] = "--use-standard-socket"; 
+            argv[2] = NULL;  
+
+            err = spawn_process_detached (agent_program, argv);
+            if (err)
+              DEBUG ("failed to start agent `%s': %s\n",
+		     agent_program, gpg_strerror (err));
+            else
+              {
+                /* Give the agent some time to prepare itself. */
+                Sleep (3 * 1000);
+                /* Now try again to connect the agent.  */
+                err = assuan_socket_connect (ctx_r, sockname, 0);
+              }
+          }
+#else /*!HAVE_W32_SYSTEM*/
+          {
+            const char *pgmname;
+            const char *argv[3];
+            int no_close_list[3];
+            int i;
+
+            if ( !(pgmname = strrchr (agent_program, '/')))
+              pgmname = agent_program;
+            else
+              pgmname++;
+            
+            argv[0] = pgmname;
+            argv[1] = "--server";
+            argv[2] = NULL;
+            
+            i=0;
+            no_close_list[i++] = fileno (stderr);
+            no_close_list[i] = -1;
+            
+            /* Connect to the agent and perform initial handshaking. */
+            err = assuan_pipe_connect (ctx_r, agent_program, argv,
+				       no_close_list);
+          }
+#endif /*!HAVE_W32_SYSTEM*/
+        }
+      free (sockname);
     }
-
-  infostr = strdup (infostr);
-  if (!infostr)
-    return gpg_error_from_errno (errno);
-
-  if (!(ptr = strchr (infostr, ':')) || ptr == infostr)
+  else
     {
-      DEBUG ("malformed GPG_AGENT_INFO environment variable");
+      int pid;
+      int protocol_version;
+
+      infostr = strdup (infostr);
+      if (!infostr)
+	return gpg_error_from_errno (errno);
+
+      if (!(ptr = strchr (infostr, PATHSEP_C)) || ptr == infostr)
+	{
+	  DEBUG ("malformed GPG_AGENT_INFO environment variable");
+	  free (infostr);
+	  force_pipe_server = 1;
+	  goto restart;
+	}
+
+      *(ptr++) = 0;
+      pid = atoi (ptr);
+      while (*ptr && *ptr != PATHSEP_C)
+	ptr++;
+      protocol_version = *ptr ? atoi (ptr + 1) : 0;
+      if (protocol_version != 1)
+	{
+	  DEBUG ("GPG agent protocol version '%d' not supported",
+		 protocol_version);
+	  free (infostr);
+	  force_pipe_server = 1;
+	  goto restart;
+	}
+      
+      err = assuan_socket_connect (ctx_r, infostr, pid);
       free (infostr);
-      return gpg_error (GPG_ERR_NO_AGENT);
-    }
-  *(ptr++) = 0;
-  pid = atoi (ptr);
-  while (*ptr && *ptr != ':')
-    ptr++;
-  protocol_version = *ptr ? atoi (ptr + 1) : 0;
-  if (protocol_version != 1)
-    {
-      DEBUG ("GPG agent protocol version '%d' not supported",
-	     protocol_version);
-      free (infostr);
-      return gpg_error (GPG_ERR_NO_AGENT);
+      if (err)
+	{
+	  DEBUG ("cannot connect to GPG agent: %s", gpg_strerror (err));
+	  force_pipe_server = 1;
+	  goto restart;
+	}
     }
 
-  err = assuan_socket_connect (ctx_r, infostr, pid);
-  free (infostr);
   if (err)
     {
       DEBUG ("cannot connect to GPG agent: %s", gpg_strerror (err));
@@ -102,6 +363,26 @@ agent_connect (assuan_context_t *ctx_r)
 }
 
 
+/* This is the default inquiry callback.  It mainly handles the
+   Pinentry notifications.  */
+static int
+default_inq_cb (void *opaque, const char *line)
+{
+  (void)opaque;
+
+  if (!strncmp (line, "PINENTRY_LAUNCHED", 17) && (line[17]==' '||!line[17]))
+    {
+      gnupg_allow_set_foregound_window ((pid_t)strtoul (line+17, NULL, 10));
+      /* We do not pass errors to avoid breaking other code.  */
+    }
+  else
+    DEBUG ("ignoring gpg-agent inquiry `%s'\n", line);
+
+  return 0;
+}
+
+
+
 /* Send a simple command to the agent.  */
 static gpg_error_t 
 agent_simple_cmd (assuan_context_t ctx, const char *fmt, ...)
@@ -118,7 +399,8 @@ agent_simple_cmd (assuan_context_t ctx, const char *fmt, ...)
   if (res < 0)
     return gpg_error_from_errno (errno);
 
-  err = assuan_transact (ctx, optstr, NULL, NULL, NULL, NULL, NULL, NULL);
+  err = assuan_transact (ctx, optstr, NULL, NULL, default_inq_cb,
+			 NULL, NULL, NULL);
   if (err)
     DEBUG ("gpg-agent command '%s' failed: %s", optstr, gpg_strerror (err));
   free (optstr);
@@ -139,6 +421,8 @@ agent_configure (assuan_context_t ctx)
   char *old_lc = NULL;
   char *dft_lc = NULL;
 #endif
+  char *dft_xauthority = NULL;
+  char *dft_pinentry_user_data = NULL;
 
   err = agent_simple_cmd (ctx, "RESET");
   if (err)
@@ -154,12 +438,12 @@ agent_configure (assuan_context_t ctx)
   dft_ttyname = getenv ("GPG_TTY");
   if ((!dft_ttyname || !*dft_ttyname) && ttyname (0))
     dft_ttyname = ttyname (0);
-  if (!dft_ttyname)
-    return 0;
-
-  err = agent_simple_cmd (ctx, "OPTION ttyname=%s", dft_ttyname);
-  if (err)
-    return err;
+  if (dft_ttyname)
+    {
+      err = agent_simple_cmd (ctx, "OPTION ttyname=%s", dft_ttyname);
+      if (err)
+	return err;
+    }
 
   dft_ttytype = getenv ("TERM");
   if (dft_ttytype)
@@ -205,6 +489,29 @@ agent_configure (assuan_context_t ctx)
     }
 #endif
 
+  dft_xauthority = getenv ("XAUTHORITY");
+  if (dft_xauthority)
+    err = agent_simple_cmd (ctx, "OPTION xauthority=%s", dft_xauthority);
+  if (gpg_err_code (err) == GPG_ERR_UNKNOWN_OPTION)
+    err = 0;
+  else if (err)
+    return err;
+
+  dft_pinentry_user_data = getenv ("PINENTRY_USER_DATA");
+  if (dft_pinentry_user_data)
+    err = agent_simple_cmd (ctx, "OPTION pinentry_user_data=%s",
+	                    dft_pinentry_user_data);
+  if (gpg_err_code (err) == GPG_ERR_UNKNOWN_OPTION)
+    err = 0;
+  else if (err)
+    return err;
+
+  err = agent_simple_cmd (ctx, "OPTION allow-pinentry-notify");
+  if (gpg_err_code (err) == GPG_ERR_UNKNOWN_OPTION)
+    err = 0;
+  else if (err)
+    return err;
+
   return err;
 }
 
@@ -497,7 +804,8 @@ scute_agent_learn (struct agent_card_info_s *info)
 
   memset (info, 0, sizeof (*info));
   err = assuan_transact (agent_ctx, "LEARN --send",
-			 NULL, NULL, NULL, NULL, learn_status_cb, info);
+			 NULL, NULL, default_inq_cb,
+			 NULL, learn_status_cb, info);
 
   return err;
 }
@@ -527,7 +835,8 @@ scute_agent_check_status (void)
   char flag = '-';
 
   err = assuan_transact (agent_ctx, "SCD GETINFO status",
-			 read_status_cb, &flag, NULL, NULL, NULL, NULL);
+			 read_status_cb, &flag, default_inq_cb,
+			 NULL, NULL, NULL);
 
   if (err)
     return err;
@@ -598,7 +907,8 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
     return gpg_error (GPG_ERR_INV_ARG);
 
   snprintf (cmd, sizeof (cmd), "SIGKEY %s", grip);
-  err = assuan_transact (agent_ctx, cmd, NULL, NULL, NULL, NULL, NULL, NULL);
+  err = assuan_transact (agent_ctx, cmd, NULL, NULL, default_inq_cb,
+			 NULL, NULL, NULL);
   if (err)
     return err;
 
@@ -607,12 +917,13 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
   pretty_data[2 * len] = '\0';
 
   snprintf (cmd, sizeof (cmd), "sethash --hash=tls-md5sha1 %s", pretty_data);
-  err = assuan_transact (agent_ctx, cmd, NULL, NULL, NULL, NULL, NULL, NULL);
+  err = assuan_transact (agent_ctx, cmd, NULL, NULL, default_inq_cb,
+			 NULL, NULL, NULL);
   if (err)
     return err;
 
   err = assuan_transact (agent_ctx, "PKSIGN",
-			 pksign_cb, &sig, NULL, NULL, NULL, NULL);
+			 pksign_cb, &sig, default_inq_cb, NULL, NULL, NULL);
   if (err)
     return err;
 
@@ -639,7 +950,8 @@ gpg_error_t scute_agent_is_trusted (char *fpr, bool *is_trusted)
   char cmd[150];
 
   snprintf (cmd, sizeof (cmd), "ISTRUSTED %s", fpr);
-  err = assuan_transact (agent_ctx, cmd, NULL, NULL, NULL, NULL, NULL, NULL);
+  err = assuan_transact (agent_ctx, cmd, NULL, NULL, default_inq_cb,
+			 NULL, NULL, NULL);
   if (err && gpg_err_code (err) != GPG_ERR_NOT_TRUSTED)
     return err;
   else if (!err)
