@@ -42,6 +42,12 @@
 
 #include "cert.h"
 #include "support.h"
+#include "debug.h"
+
+
+#ifndef HAVE_W32_SYSTEM
+#define COMPAT_FALLBACK
+#endif
 
 
 /* The maximum length of a key listing line.  We take the double of
@@ -128,7 +134,14 @@ parse_timestamp (const char *timestamp, char **endp)
 
       if (endp)
         *endp = (char*)(timestamp + 15);
+#ifdef HAVE_TIMEGM
       return timegm (&buf);
+#else
+      /* FIXME: Need to set TZ to UTC, but that is not
+	 thread-safe.  */
+      return mktime (&buf);
+#endif
+
     }
   else
     return (time_t)strtoul (timestamp, endp, 10);
@@ -519,6 +532,7 @@ struct search_ctx_by_field
 };
   
 
+#ifdef COMPAT_FALLBACK
 /* This is a compatibility function for GPGSM 2.0.0, which does not
    support the --data option with the EXPORT command.  */
 static gpg_error_t
@@ -544,11 +558,13 @@ export_cert_compat (char *fpr, struct cert *cert)
   child_fds[0] = output_fds[1];
   child_fds[1] = -1;
 
-  err = assuan_pipe_connect (&ctx, GPGSM_PATH, argv, child_fds);
+  err = assuan_pipe_connect_ext (&ctx, get_gpgsm_path (), argv, child_fds,
+				 NULL, NULL, 128);
   close (output_fds[1]);
   if (err)
     {
       close (output_fds[0]);
+      DEBUG ("failed to spawn %s\n", get_gpgsm_path ());
       return err;
     }
 
@@ -581,6 +597,7 @@ export_cert_compat (char *fpr, struct cert *cert)
   close (output_fds[0]);
   return err;
 }
+#endif
 
 
 struct export_hook
@@ -630,21 +647,13 @@ export_cert (char *fpr, struct cert *cert)
   const char *argv[] = { "gpgsm", "--server", NULL };
 #define COMMANDLINELEN 80
   char cmd[COMMANDLINELEN];
-  int output_fds[2];
-  int child_fds[2];
   struct export_hook exp;
 
-  if(pipe (output_fds) < 0)
-    return gpg_error_from_syserror ();
-
-  child_fds[0] = output_fds[1];
-  child_fds[1] = -1;
-
-  err = assuan_pipe_connect (&ctx, GPGSM_PATH, argv, child_fds);
-  close (output_fds[1]);
+  err = assuan_pipe_connect_ext (&ctx, get_gpgsm_path (), argv, NULL,
+				 NULL, NULL, 128);
   if (err)
     {
-      close (output_fds[0]);
+      DEBUG ("spawning %s\n", get_gpgsm_path ());
       return err;
     }
 
@@ -656,13 +665,13 @@ export_cert (char *fpr, struct cert *cert)
   err = assuan_transact (ctx, cmd, export_cert_cb, &exp,
 			 NULL, NULL, NULL, NULL);
   assuan_disconnect (ctx);
-  close (output_fds[0]);
 
   if (!err)
     {
       cert->cert_der = exp.buffer;
       cert->cert_der_len = exp.buffer_len;
     }
+#ifdef COMPAT_FALLBACK
   else if (gpg_err_code (err) == GPG_ERR_ASS_NO_OUTPUT)
     {
       /* For compatibility with GPGSM 2.0.0, we fall back to a work
@@ -674,6 +683,7 @@ export_cert (char *fpr, struct cert *cert)
 	}
       err = export_cert_compat (fpr, cert);
     }
+#endif
 
   if (!err)
     err = scute_agent_is_trusted (fpr, &cert->is_trusted);
@@ -717,9 +727,13 @@ scute_gpgsm_search_certs_by_grip (const char *grip,
   const char *argv[] = { "gpgsm", "--server", NULL };
   struct search_ctx_by_field search;
 
-  err = assuan_pipe_connect (&ctx, GPGSM_PATH, argv, NULL);
+  err = assuan_pipe_connect_ext (&ctx, get_gpgsm_path (), argv, NULL,
+				 NULL, NULL, 128);
   if (err)
-    return err;
+    {
+      DEBUG ("spawning %s\n", get_gpgsm_path ());
+      return err;
+    }
 
   search.field = SEARCH_BY_GRIP;
   search.pattern = grip;
@@ -744,9 +758,13 @@ scute_gpgsm_search_certs_by_fpr (const char *fpr,
   const char *argv[] = { "gpgsm", "--server", NULL };
   struct search_ctx_by_field search;
 
-  err = assuan_pipe_connect (&ctx, GPGSM_PATH, argv, NULL);
+  err = assuan_pipe_connect_ext (&ctx, get_gpgsm_path (), argv, NULL,
+				 NULL, NULL, 128);
   if (err)
-    return err;
+    {
+      DEBUG ("failed to spawn %s\n", get_gpgsm_path ());
+      return err;
+    }
 
   search.field = SEARCH_BY_FPR;
   search.pattern = fpr;
